﻿open LambdaCalculus
open Parser
open FParsec

let rec repl() = 
    printf "> "
    let input = System.Console.ReadLine().Trim()
    if input <> "" && input <> ":q" then
        match run ExtendedLambdaParser.fullTerm (LambdaCalculus.ydef input) with
            | Success(term, _, _)  -> 
                let basicTerm = ExtendedLambdaParser.rewrite term
                printfn "%s\n" (prettyPrint (eval basicTerm))
            | Failure(msg, _, _) -> printfn "Failure %A" msg
        repl()
    else
        ()

let runFile fn =
    let txt = System.IO.File.ReadAllText(fn)
    let term = run ExtendedLambdaParser.fullTerm txt

    match term with
    | Success(term, _, _)  -> 
        let basicTerm = ExtendedLambdaParser.rewrite term
        let res = Interpreter.dechurch (Interpreter.eval basicTerm Interpreter.Global)
        printfn "%i" res
    | Failure(msg, _, _) -> printfn "Failure %A" msg

let runFileClassic fn =
    let txt = System.IO.File.ReadAllText(fn)
    let term = run fullTerm txt

    match term with
    | Success(term, _, _)  -> 
        let res = Interpreter.dechurch (Interpreter.eval term Interpreter.Global)
        printfn "%i" res
    | Failure(msg, _, _) -> printfn "Failure %A" msg
 
let compileFile fn =
    let txt = System.IO.File.ReadAllText(fn)
    match run ExtendedLambdaParser.fullTerm txt with
        | Success(term, _, _)  -> 
            let basicTerm = ExtendedLambdaParser.rewrite term
            System.IO.File.WriteAllText(fn + ".classic", prettyPrint basicTerm)
        | Failure(msg, _, _) -> printfn "Failure %A" msg

let compileJSFile fn =
    let txt = System.IO.File.ReadAllText(fn)
    match run ExtendedLambdaParser.fullTerm txt with
        | Success(term, _, _)  -> 
            let basicTerm = ExtendedLambdaParser.rewrite term
            let jsSource = makeJS basicTerm
            let jsSourceEx = sprintf "function run() {var t = %s; var cnt=0; (t(function(_) { cnt += 1; }))(function(_){return 0;}); alert(cnt);}" jsSource

            let source = sprintf "<html><head><script lang=\"javascript\">%s</script></head><body><a href=\"javascript:run();\">Run script</a></body></html>" jsSourceEx

            System.IO.File.WriteAllText(fn + ".html", source)
        | Failure(msg, _, _) -> printfn "Failure %A" msg

let main() =
    match System.Environment.GetCommandLineArgs() with
    | [| _ |] -> repl()
    | [| _; fn |] -> runFile fn
    | [| _; "-classic"; fn |] -> runFileClassic fn
    | [| _; "-compile"; fn |] -> compileFile fn
    | [| _; "-js"; fn |]      -> compileJSFile fn
    | _ ->
        printfn "Lambda term interpreter \n Options: <Filename>, -classic, -compile, -js"
        repl()

main()